DROP TRIGGER IF EXISTS update_cache;

DELIMITER |

CREATE TRIGGER update_cache AFTER INSERT ON ulog
    FOR EACH ROW BEGIN

        /* Dropped packets */
        IF NEW.state=0 OR NEW.state IS NULL
        THEN

            /* Offenders */
            IF EXISTS(SELECT * FROM offenders WHERE ip_addr = NEW.ip_saddr)
            THEN
                UPDATE offenders SET count = count + 1, last_time = UNIX_TIMESTAMP() WHERE ip_addr = NEW.ip_saddr;
            ELSE
                INSERT INTO offenders VALUES (NEW.ip_saddr, UNIX_TIMESTAMP(), UNIX_TIMESTAMP(), 1);
            END IF;

            /* TCP_PORTS */
            IF NEW.tcp_dport IS NOT NULL
            THEN
                IF EXISTS(SELECT * FROM tcp_ports WHERE tcp_dport = NEW.tcp_dport)
                THEN
                    UPDATE tcp_ports SET count = count + 1, last_time = UNIX_TIMESTAMP() WHERE tcp_dport = NEW.tcp_dport;
                ELSE
                    INSERT INTO tcp_ports VALUES (NEW.tcp_dport, UNIX_TIMESTAMP(), UNIX_TIMESTAMP(), 1);
                END IF;
            END IF;

            /* UDP_PORTS */
            IF NEW.udp_dport IS NOT NULL
            THEN
                IF EXISTS(SELECT * FROM udp_ports WHERE udp_dport = NEW.udp_dport)
                THEN
                    UPDATE udp_ports SET count = count + 1, last_time = UNIX_TIMESTAMP() WHERE udp_dport = NEW.udp_dport;
                ELSE
                    INSERT INTO udp_ports VALUES (NEW.udp_dport, UNIX_TIMESTAMP(), UNIX_TIMESTAMP(), 1);
                END IF;
            END IF;

            /* USERSSTATS (dropped) */
            IF NEW.user_id IS NOT NULL
            THEN
                IF EXISTS(SELECT * FROM usersstats WHERE user_id = NEW.user_id)
                THEN
                    UPDATE usersstats SET bad_conns = bad_conns + 1, last_time = UNIX_TIMESTAMP() WHERE user_id = NEW.user_id;
                ELSE /* bad_conns=1, good_conns=0 */
                    INSERT INTO usersstats VALUES (NEW.user_id, NEW.username, 1, 0,  UNIX_TIMESTAMP(), UNIX_TIMESTAMP());
                END IF;
            END IF;

        /* Allowed packet */
        ELSE

            /* USERSSTATS (allowed) */
            IF NEW.user_id IS NOT NULL
            THEN
                IF EXISTS(SELECT * FROM usersstats WHERE user_id = NEW.user_id)
                THEN
                    UPDATE usersstats SET good_conns = good_conns + 1, last_time = UNIX_TIMESTAMP() WHERE user_id = NEW.user_id;
                ELSE /* bad_conns=0, good_conns=1 */
                    INSERT INTO usersstats VALUES (NEW.user_id, NEW.username, 0, 1, UNIX_TIMESTAMP(), UNIX_TIMESTAMP());
                END IF;
            END IF;

        END IF;
    END;
|

DELIMITER ;
